# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import logging
import xml.etree.cElementTree as ET

from driverng import Message, utils

logger = logging.getLogger("driverng.drivers.mylaxml")


class Driver(Message):
    SPEC = {"code": "mylaxml", "label": "Maestria/Myla XML"}
    ENCODING = "utf8"

    OPTIONS = [
        Message.Option("bmr", label="Champ bactérie multirésistante"),
        Message.Option("default_cmi", label="CMI par défaut", default="-"),
        Message.BooleanOption(
            "myla_ignore_unk",
            label="Ignorer les résultats de type unk en hémoculture",
            default=False,
        ),
        Message.BooleanOption(
            "hemoc_temps_positivite",
            label="Récupérer le délai uniquement pour les HEMOC +",
            default=False,
        ),
        Message.Option(
            "hemoc_delai_incubation",
            label="Saisir le délai par défaut d'incubation pour l'hémoculture",
            default="",
        ),
        Message.Option(
            "myla_prefixe_germe",
            label="Préfixe des textes codés des germes (par défaut G)",
            default="G",
        ),
        Message.Option(
            "myla_prefixe_antibio",
            label="Préfixe des textes codés des antibiotiques (par défaut A)",
            default="A",
        ),
        Message.BooleanOption(
            "atb_commentaire_automatique",
            label="Récupérer les commentaires automatisés (balise xml userAstCommentList",
            default=False,
        ),
        Message.BooleanOption(
            "myla_germe_no_upper",
            label="Ne pas mettre le code du germe en majuscule",
            default=False,
        ),
        Message.BooleanOption(
            "myla_antibiotic_no_upper",
            label="Ne pas mettre les codes des antibiotiques en majuscule",
            default=False,
        ),
        Message.BooleanOption(
            "myla_ignore_suppressed_drug",
            label="Ignorer les antibiotiques supprimés (balise suppressedDrug)",
            default=False,
        ),
        Message.BooleanOption(
            "myla_send_service_xml_location",
            label="Envoyer le code du service dans la balise xml location",
            default=False,
        ),
        Message.Option(
            "myla_prefixe_atb_adagio",
            label="Préfixe pour les antibiotiques réalisés par l'ADAGIO (via MYLA)",
            default="",
        ),
        Message.BooleanOption(
            "myla_diameter", label="Intégrer les diamètres dans les ATB", default=False
        ),
        Message.BooleanOption(
            "myla_commentaire_user",
            label="Récupérer les commentaires utilisateur (balise xml userComment/comment/text)",
            default=False,
        ),
        Message.BooleanOption(
            "myla_instrument_name",
            label="Récupérer instrumentName au lieu de serialNumber dans le champ analyseur",
            default=False,
        ),
        Message.BooleanOption(
            "myla_send_atb_to_myla",
            label="Envoyer les antibiogrammes vers le Myla",
            default=False,
        ),
    ]

    def __init__(
        self,
        bmr="",
        default_cmi="-",
        myla_ignore_unk=False,
        hemoc_temps_positivite=False,
        hemoc_delai_incubation="",
        myla_prefixe_germe="G",
        myla_prefixe_antibio="A",
        atb_commentaire_automatique=False,
        myla_germe_no_upper=False,
        myla_antibiotic_no_upper=False,
        myla_ignore_suppressed_drug=False,
        myla_send_service_xml_location=False,
        myla_prefixe_atb_adagio="",
        myla_diameter=False,
        **kwargs
    ):
        self.bmr = bmr or ""
        self.default_cmi = default_cmi or "-"
        self.myla_ignore_unk = myla_ignore_unk
        self.hemoc_temps_positivite = hemoc_temps_positivite
        self.hemoc_delai_incubation = hemoc_delai_incubation
        self.prefixe_germe = myla_prefixe_germe
        self.prefixe_antibio = myla_prefixe_antibio
        self.atb_commentaire_automatique = atb_commentaire_automatique
        self.myla_germe_no_upper = myla_germe_no_upper
        self.myla_antibiotic_no_upper = myla_antibiotic_no_upper
        self.myla_ignore_suppressed_drug = myla_ignore_suppressed_drug
        self.myla_send_service_xml_location = myla_send_service_xml_location
        self.myla_prefixe_atb_adagio = myla_prefixe_atb_adagio
        self.myla_diameter = myla_diameter
        self.myla_commentaire_user = kwargs.pop("myla_commentaire_user", False)
        self.myla_instrument_name = kwargs.pop("myla_instrument_name", False)
        self.myla_send_atb_to_myla = kwargs.pop("myla_send_atb_to_myla", False)
        return super(Driver, self).__init__(**kwargs)

    def parse(self, data):
        return ET.fromstring(data.encode("utf8"))

    def to_string(self, message):
        return utils.prettify_xml(ET.tostring(message, "utf-8"))

    def get_message_type(self, message):
        return self.ORDER

    def get_order_message(self, worklist, context):
        root = ET.Element("lisMessage")
        header = ET.SubElement(root, "header")
        ET.SubElement(header, "version").text = "1394-97"
        ET.SubElement(header, "dateTime").text = utils.format_date(
            context["now"], fmt="%Y%m%d%H%M%S"
        )

        empty = True
        for sample, _ in worklist.get_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: None},
        ):
            request = ET.SubElement(root, "request")
            patient = ET.SubElement(request, "patientInformation")
            ET.SubElement(patient, "patientIdentifier").text = sample["patient_id"]
            if sample["alternate_patient_id"]:
                ET.SubElement(patient, "patientVisitNumber").text = sample[
                    "alternate_patient_id"
                ]
            ET.SubElement(patient, "firstName").text = sample["first_name"]
            ET.SubElement(patient, "lastName").text = sample["last_name"]
            if sample["birthdate"]:
                ET.SubElement(patient, "birthdate").text = utils.format_date(
                    sample["birthdate"], fmt="%Y%m%d"
                )
            ET.SubElement(patient, "sex").text = sample["sex"]
            if sample["physician"]:
                ET.SubElement(patient, "physicianIdentifier").text = sample["physician"]
            if sample["physician_name"]:
                ET.SubElement(patient, "physicianName").text = sample["physician_name"]
            if sample.get("admission_date"):
                ET.SubElement(patient, "admissionDate").text = utils.format_date(
                    sample.get("admission_date"), fmt="%Y%m%d"
                )
            if sample["room"] and not self.myla_send_service_xml_location:
                ET.SubElement(patient, "location").text = sample["room"]
                ET.SubElement(patient, "locationText").text = sample["room"]
            elif sample["service"] and self.myla_send_service_xml_location:
                ET.SubElement(patient, "location").text = sample["service"]
                ET.SubElement(patient, "locationText").text = sample["service"]
            if sample["hospital_service"]:
                ET.SubElement(patient, "hospitalService").text = sample[
                    "hospital_service"
                ]
                ET.SubElement(patient, "hospitalServiceText").text = sample[
                    "hospital_service"
                ]
            elif sample["service"]:
                ET.SubElement(patient, "hospitalService").text = sample["service"]
                if sample["service_name"]:
                    ET.SubElement(patient, "hospitalServiceText").text = sample[
                        "service_name"
                    ]
                else:
                    ET.SubElement(patient, "hospitalServiceText").text = sample[
                        "service"
                    ]
            if sample["lab"]:
                ET.SubElement(patient, "hospitalInstitution").text = sample["lab"]
                ET.SubElement(patient, "hospitalInstitutionText").text = sample["lab"]
            identification_germ_to_send = False
            atb_to_send = False
            for result in sample["results"]:
                if result["resu_json"] and "germes" in result["resu_json"]:
                    identification_germ_to_send = True
                elif self.myla_send_atb_to_myla and result.get("atb"):
                    atb_to_send = True
            if identification_germ_to_send or atb_to_send:
                # Identification de germe à envoyer. Il faut envoyer une
                # identification par test et testOrder
                for result in sample["results"]:
                    if result.get("atb"):
                        test_order = ET.SubElement(request, "testOrder")
                        specimen = ET.SubElement(test_order, "specimen")
                        specimen_nature = ""
                        specimen_source = ""
                        ET.SubElement(specimen, "specimenIdentifier").text = sample[
                            "sample_id"
                        ]
                        for order in sample["orders"]:
                            ET.SubElement(specimen, "specimenType").text = order["code"]
                            ET.SubElement(specimen, "specimenTypeText").text = order[
                                "code"
                            ]
                            ET.SubElement(specimen, "specimenNature").text = order[
                                "code"
                            ]
                            ET.SubElement(specimen, "specimenNatureText").text = order[
                                "code"
                            ]
                            if sample["collection_date"]:
                                ET.SubElement(
                                    specimen, "collectedDateTime"
                                ).text = utils.format_date(
                                    sample["collection_date"], fmt="%Y%m%d%H%M%S"
                                )
                        rank_germ = result["sub_code"]
                        id_germ = result["value"]
                        ET.SubElement(test_order, "isolateNumber").text = str(
                            rank_germ if rank_germ is None else ""
                        )
                        test = ET.SubElement(test_order, "test")
                        universal_identifier = ET.SubElement(
                            test, "universalIdentifier"
                        )
                        ET.SubElement(
                            universal_identifier, "testIdentifier"
                        ).text = "ID"
                        if id_germ:
                            # Bloc identification de germe
                            ET.SubElement(
                                universal_identifier, "testGroupCode"
                            ).text = "ID"
                            # Encodage de l'identification du germe
                            resu = ET.SubElement(test, "result")
                            value = ET.SubElement(resu, "value")
                            identification = ET.SubElement(value, "identification")
                            significant_taxon = ET.SubElement(
                                identification, "significantTaxon"
                            )
                            germ_prefix = "{}_".format(self.prefixe_germe)
                            ET.SubElement(significant_taxon, "code").text = (
                                id_germ[2:]
                                if id_germ
                                and germ_prefix
                                and id_germ.startswith(germ_prefix)
                                else id_germ
                            )
                            ET.SubElement(significant_taxon, "name").text = result[
                                "value_label"
                            ]

                        # ATB
                        ast = ET.SubElement(value, "ast")
                        for atb in result["atb"]:
                            antibiotic = ET.SubElement(ast, "antibiotic")
                            code_atb = atb.get("ANTIBIO")
                            ET.SubElement(antibiotic, "code").text = (
                                code_atb[2:] if code_atb.startswith("A_") else code_atb
                            )
                            ET.SubElement(antibiotic, "name").text = atb.get(
                                "LABEL_ANTIBIO"
                            )
                            highmic = ET.SubElement(antibiotic, "highMic")
                            if atb.get("CMI"):
                                cmi = atb.get("CMI")
                                if (cmi.startswith("<=") and cmi[:2] == "<=") or (
                                    cmi.startswith(">=") and cmi[:2] == ">="
                                ):
                                    ET.SubElement(highmic, "micSign").text = cmi[:2]
                                    cmi = cmi[2:]
                                elif (cmi.startswith(">") and cmi[:2] != ">=") or (
                                    cmi.startswith("<") and cmi[:2] != "<="
                                ):
                                    ET.SubElement(highmic, "micSign").text = cmi[:1]
                                    cmi = cmi[1:]
                                    ET.SubElement(highmic, "micValue").text = cmi
                            ET.SubElement(antibiotic, "category").text = atb.get(
                                "SENSI", ""
                            )
                            ET.SubElement(
                                antibiotic, "nonExpertCategory"
                            ).text = atb.get("SENSI", "")
                        empty = False
                    elif (
                        identification_germ_to_send
                        and result["resu_json"]
                        and "germes" in result["resu_json"]
                    ):
                        for resu_json_germe in result["resu_json"]["germes"]:
                            if not isinstance(resu_json_germe, dict):
                                continue
                            test_order = ET.SubElement(request, "testOrder")
                            specimen = ET.SubElement(test_order, "specimen")
                            specimen_nature = ""
                            specimen_source = ""
                            ET.SubElement(specimen, "specimenIdentifier").text = sample[
                                "sample_id"
                            ]
                            for order in sample["orders"]:
                                ET.SubElement(specimen, "specimenType").text = order[
                                    "code"
                                ]
                                ET.SubElement(
                                    specimen, "specimenTypeText"
                                ).text = order["code"]
                                ET.SubElement(specimen, "specimenNature").text = order[
                                    "code"
                                ]
                                ET.SubElement(
                                    specimen, "specimenNatureText"
                                ).text = order["code"]
                                if sample["collection_date"]:
                                    ET.SubElement(
                                        specimen, "collectedDateTime"
                                    ).text = utils.format_date(
                                        sample["collection_date"], fmt="%Y%m%d%H%M%S"
                                    )
                            if identification_germ_to_send:
                                try:
                                    rank_germ = resu_json_germe.get("rang")
                                except KeyError:
                                    rank_germ = ""
                                try:
                                    key_germe = [
                                        cle
                                        for cle in resu_json_germe.keys()
                                        if cle.lower() == "id"
                                    ]
                                    id_germ = resu_json_germe.get(
                                        key_germe[0] if key_germe else None
                                    )
                                    if isinstance(id_germ, list):
                                        id_germ = ""
                                except KeyError:
                                    id_germ = ""
                            if id_germ and rank_germ:
                                # Encodage du rang du germe identifié
                                ET.SubElement(test_order, "isolateNumber").text = str(
                                    rank_germ
                                )
                                rank_germ = ""
                            test = ET.SubElement(test_order, "test")
                            universal_identifier = ET.SubElement(
                                test, "universalIdentifier"
                            )
                            if id_germ:
                                # Bloc identification de germe
                                ET.SubElement(
                                    universal_identifier, "testGroupCode"
                                ).text = "ID"
                                # Encodage de l'identification du germe
                                resu = ET.SubElement(test, "result")
                                value = ET.SubElement(resu, "value")
                                identification = ET.SubElement(value, "identification")
                                significant_taxon = ET.SubElement(
                                    identification, "significantTaxon"
                                )
                                germ_prefix = "{}_".format(self.prefixe_germe)
                                ET.SubElement(significant_taxon, "code").text = (
                                    id_germ[2:]
                                    if id_germ
                                    and germ_prefix
                                    and id_germ.startswith(germ_prefix)
                                    else id_germ
                                )
                                id_germ = ""
                            empty = False
            else:
                for order in sample["orders"]:
                    test_order = ET.SubElement(request, "testOrder")
                    specimen = ET.SubElement(test_order, "specimen")
                    specimen_nature = ""
                    specimen_source = ""
                    for result in sample["results"]:
                        if result["code"].upper().startswith("SPECIMENNATURE_"):
                            specimen_nature = result["value"]
                        elif result["code"].upper().startswith("SPECIMENSOURCE_"):
                            specimen_source = result["value"]
                    ET.SubElement(specimen, "specimenIdentifier").text = sample[
                        "sample_id"
                    ]

                    # Code ou Type de prélèvement
                    prelevement_type = order["code"]
                    delai_incubation_hemoculture = False
                    for result in sample["results"]:
                        if sample.specimen == "BC" and result["code"].startswith(
                            "EXPECTED_DURATION"
                        ):
                            delai_incubation_hemoculture = True
                            prelevement_type = sample.specimen
                            continue
                    if not delai_incubation_hemoculture:
                        tcode = order["code"].upper().split("_")
                        if len(tcode) > 1:
                            prelevement_type = tcode[1]
                    if prelevement_type:
                        ET.SubElement(specimen, "specimenType").text = prelevement_type
                        ET.SubElement(
                            specimen, "specimenTypeText"
                        ).text = prelevement_type

                    # Nature du prélèvement
                    if specimen_nature:
                        ET.SubElement(specimen, "specimenNature").text = specimen_nature
                        ET.SubElement(
                            specimen, "specimenNatureText"
                        ).text = specimen_nature
                    else:
                        ET.SubElement(specimen, "specimenNature").text = order["code"]
                        ET.SubElement(specimen, "specimenNatureText").text = order[
                            "code"
                        ]

                    # Localisation ou source de prélèvement
                    if specimen_source:
                        ET.SubElement(specimen, "specimenSource").text = specimen_source
                        ET.SubElement(
                            specimen, "specimenSourceText"
                        ).text = specimen_source
                    if sample["collection_date"]:
                        ET.SubElement(
                            specimen, "collectedDateTime"
                        ).text = utils.format_date(
                            sample["collection_date"], fmt="%Y%m%d%H%M%S"
                        )

                    test = ET.SubElement(test_order, "test")
                    universal_identifier = ET.SubElement(test, "universalIdentifier")

                    # Type de prélèvement d'hémoculture
                    if sample["specimen"] == "SU":
                        ET.SubElement(
                            universal_identifier, "testGroupCode"
                        ).text = sample["specimen"]
                    elif sample["specimen"] == "ID":
                        ET.SubElement(
                            universal_identifier, "testGroupCode"
                        ).text = sample["specimen"]
                    elif sample["specimen"] == "BC":
                        ET.SubElement(
                            universal_identifier, "testGroupCode"
                        ).text = sample["specimen"]
                        delai_duration = False
                        for result in sample["results"]:
                            # Transmettre uniquement le premier test de délai
                            # d'incubation
                            if result["code"].startswith("EXPECTED_DURATION"):
                                ET.SubElement(test, "expectedDuration").text = result[
                                    "value"
                                ]
                                delai_duration = True
                                continue
                        if self.hemoc_delai_incubation and not delai_duration:
                            ET.SubElement(
                                test, "expectedDuration"
                            ).text = self.hemoc_delai_incubation
                    empty = False
        if not empty:
            return root

    def get_result_dict(self, message):
        samples_results = {}
        if message.find("header"):
            if (
                message.find("header").find("serialNumber") is None
                and not self.myla_instrument_name
            ):
                analyseur = ""
            elif not self.myla_instrument_name:
                analyseur = message.find("header").find("serialNumber").text
            else:
                analyseur = message.find("header").find("instrumentName").text
        else:
            analyseur = ""
        for request in message.findall("requestResult"):
            for test_order in request.findall("testOrder"):
                sample_id = test_order.findtext("./specimen/specimenIdentifier")
                sample_results = samples_results.setdefault(sample_id, {})
                specimen_type = test_order.findtext("./specimen/specimenType")
                lot_number = test_order.findtext(
                    "./test/result/bmxResultData/disposable/lotNumber"
                )

                # Bloodculture bottle
                if (
                    test_order.findtext(
                        "./test/result/bmxResultData/disposable/disposableType"
                    )
                    == "BOTTLE"
                ):
                    reference = test_order.findtext(
                        "./test/result/bmxResultData/disposable/reference"
                    )
                    instrument_name = "{:.20s}".format(
                        test_order.findtext("./test/instrumentName")
                    )
                    date = datetime.datetime.strptime(
                        test_order.findtext(
                            "./test/result/value/bloodCulture/dateResultDetermined"
                        ),
                        "%Y%m%d%H%M%S",
                    )

                    # Section ID bloodculture bottle
                    test = reference + "_SECTION-ID"
                    sample_results[test] = {
                        "test": test,
                        "resu": test_order.findtext("./test/instrumentSectionId"),
                        "date": date,
                        "analyseur": instrument_name,
                    }
                    # Identification bloodculture bottle
                    test = reference + "_IDENT"
                    sample_results[test] = {
                        "test": test,
                        "resu": test_order.findtext(
                            "./test/result/bmxResultData/disposable/disposableIdentifier"
                        ),
                        "date": date,
                        "analyseur": instrument_name,
                    }
                    # Traitement du volume
                    volume = test_order.findtext("./test/result/bmxResultData/volume")
                    if volume:
                        test = reference + "_VOLUME"
                        sample_results[test] = {
                            "test": test,
                            "resu": volume,
                            "date": date,
                            "analyseur": instrument_name,
                        }
                    # Dates
                    for suffix, value in (
                        (
                            "_START_DATETIME",
                            test_order.findtext("./test/result/startDateTime"),
                        ),
                        (
                            "_DISPOSABLE_START_DATETIME",
                            test_order.findtext(
                                "./test/result/bmxResultData/disposable/disposableStartDateTime"
                            ),
                        ),
                        (
                            "_DISPOSABLE_FINAL_DATETIME",
                            test_order.findtext(
                                "./test/result/bmxResultData/disposable/disposableFinalDateTime"
                            ),
                        ),
                    ):
                        if value is not None:
                            test = reference + suffix
                            sample_results[test] = {
                                "test": test,
                                "resu": value,
                                "analyseur": instrument_name,
                            }
                    # Type de flacon
                    sample_results["BOTTLE_TYPE"] = {
                        "test": "BOTTLE_TYPE",
                        "resu": reference,
                        "analyseur": instrument_name,
                    }

                    # Ignoré dans les messages d'acquittement du tube
                    if test_order.findtext("./test/result/resultStatus") != "I":
                        # BloodCulture result
                        resu = test_order.findtext(
                            "./test/result/value/bloodCulture/result"
                        )
                        # Si l'option est activée, on ignore le résultat
                        # d'hémoculture de type unk
                        # Le Myla envoie ce type de résultat lorsqu'il ne sait
                        # pas donner un résultat. Impossible sur le MYLA de ne
                        # pas envoyer ce résultat
                        if self.myla_ignore_unk and resu == "Unk":
                            continue
                        test = reference + "_RESULT"
                        operator = test_order.findtext(
                            "./test/result/bmxResultData/disposable/disposableSetupTechnologist"
                        )
                        sample_results[test] = {
                            "test": test,
                            "resu": resu,
                            "analyseur": instrument_name,
                            "operator": operator if operator else "",
                            "date": date,
                        }
                        # Abnormal flag
                        if test_order.findtext("./test/result/abnormalFlags"):
                            test = reference + "_ABNORMAL_FLAG"
                            sample_results[test] = {
                                "test": test,
                                "resu": test_order.findtext(
                                    "./test/result/abnormalFlags"
                                ),
                                "analyseur": instrument_name,
                            }
                        # Dates
                        for suffix, value in (
                            (
                                "_TIME_TO_DETECTION",
                                test_order.findtext(
                                    "./test/result/value/bloodCulture/timeToDetectionInHours"
                                ),
                            ),
                            (
                                "_DETERMINED_DATETIME",
                                test_order.findtext(
                                    "./test/result/value/bloodCulture/dateResultDetermined"
                                ),
                            ),
                        ):
                            if (
                                self.hemoc_temps_positivite
                                and suffix == "_TIME_TO_DETECTION"
                                and test_order.findtext(
                                    "./test/result/value/bloodCulture/result"
                                )
                                != "Pos"
                            ):
                                continue
                            if suffix == "_TIME_TO_DETECTION":
                                test = reference + "_TIME_TO_DETECTION_POSITIVITY"
                                try:
                                    value_in_minutes = float(value) * 60
                                except ValueError:
                                    logger.warning(
                                        'Le délai dans la balise "timeToDetectionInHours" n\'est pas une valeur numérique'
                                    )
                                    continue
                                sample_results[test] = {
                                    "test": test,
                                    "resu": "{:02d}:{:02d}".format(
                                        int(value_in_minutes // 60),
                                        int(value_in_minutes % 60),
                                    ),
                                    "analyseur": instrument_name,
                                }
                            if value is not None:
                                test = reference + suffix
                                sample_results[test] = {
                                    "test": test,
                                    "resu": value,
                                    "analyseur": instrument_name,
                                }
                elif specimen_type:
                    if self.myla_germe_no_upper:
                        # Ne pas mettre le code du germe en majuscule
                        germe = "{}_{}".format(
                            self.prefixe_germe,
                            test_order.findtext(
                                "./test/result/value/identification/significantTaxon/code"
                            ),
                        )
                    else:
                        # Mise en majuscule du code du germe. Comportement par
                        # défaut afin de conserver la compatibilité avec les
                        # clients ayant les textes codés des germes paramétrés
                        # en majuscule
                        germe = "{}_{}".format(
                            self.prefixe_germe,
                            test_order.findtext(
                                "./test/result/value/identification/significantTaxon/code"
                            ).upper(),
                        )
                    # Traitement des commentaires automatisés des
                    # antibiogrammes
                    comment = []
                    commentaires = []
                    phenotypes = []
                    for resistance_mechanism in test_order.findall(
                        "./test/result/value/expertFinding/resistanceMechanism"
                    ):
                        drug_family = resistance_mechanism.findtext("drugFamily")
                        for phenotype in resistance_mechanism.findall("phenotype"):
                            drug_family_phenotype = "{}-{}".format(
                                drug_family, phenotype.text
                            )
                            phenotypes.append(drug_family_phenotype)
                    for ast_comment in test_order.findall(
                        "./test/result/value/ast/userAstCommentList/comment"
                    ):
                        if self.atb_commentaire_automatique:
                            comment.append(ast_comment.find(".").text)
                    if self.myla_commentaire_user:
                        for user_comment in test_order.findall(
                            "./test/result/userComment/comment"
                        ):
                            com_texte = user_comment.findtext("text")
                            if com_texte:
                                commentaires.append(com_texte)
                    result = {
                        "atb": [],
                        "seq": test_order.findtext("isolateNumber"),
                        "test": specimen_type,
                        "date": datetime.datetime.strptime(
                            test_order.findtext("./test/result/resultDateTime"),
                            "%Y%m%d%H%M%S",
                        ),
                        "resu": germe,
                        "phenotypes": phenotypes,
                        "commentaires": commentaires or comment,
                        "analyseur": analyseur,
                        "nom_antibiogramme": test_order.findtext(
                            "./test/universalIdentifier/testIdentifier"
                        ),
                    }
                    # Gestion des bacteries multi resistantes On le fait passer
                    # en antibiotique dans le même test mais ça n'a rien à voir
                    for resistance in test_order.findall(
                        "./test/result/value/expertFinding/resistanceMechanism"
                    ):
                        if resistance.findtext("drugFamily") == self.bmr:
                            bmrs = [e.text for e in resistance.findall("phenotype")]
                            # À supprimer après avoir repensé la gestion
                            # des BMR.  Spécifique Labomaine - A voir avec damien@
                            if (
                                germe != "G_STAAUR"
                                and "MODIFICATION DES PLP (mecA)" in bmrs
                            ):
                                bmrs.remove("MODIFICATION DES PLP (mecA)")
                            if bmrs:
                                result["atb"].append(
                                    (
                                        (
                                            "ANTIBIO",
                                            ";".join(["BMR_%s" % (e,) for e in bmrs]),
                                        ),
                                        ("CMI", "#"),
                                        ("SENSI", "BMR"),
                                    )
                                )

                    if comment:
                        sample_results["ATB_BIOART"] = {
                            "test": "ATB_BIOART",
                            "resu": comment,
                        }

                    # Traitement des antibiotiques
                    for antibiotic in test_order.findall(
                        "./test/result/value/ast/antibiotic"
                    ):
                        if (
                            self.myla_ignore_suppressed_drug
                            and antibiotic.findtext("suppressedDrug")
                            and antibiotic.find("suppressedDrug").text == "true"
                        ):
                            continue
                        if self.myla_antibiotic_no_upper:
                            antibiotic_code = antibiotic.findtext("code")
                        else:
                            antibiotic_code = antibiotic.findtext("code").upper()
                        if analyseur == "ADAGIO" and self.myla_prefixe_atb_adagio:
                            code = "{}_{}".format(
                                self.myla_prefixe_atb_adagio, antibiotic_code
                            )
                        elif self.prefixe_antibio:
                            code = "{}_{}".format(self.prefixe_antibio, antibiotic_code)
                        else:
                            code = antibiotic_code
                        cmi = "{}{}".format(
                            antibiotic.findtext("./highMic/micSign") or "",
                            antibiotic.findtext("./highMic/micValue") or "",
                        )
                        sensi = antibiotic.findtext("category")
                        # Ignorer les résultats de cmi et de sensibilité non
                        # testés
                        if cmi == "(-)" or sensi == "(-)":
                            continue
                        if self.myla_diameter and antibiotic.findtext("diameter"):
                            result["atb"].append(
                                (
                                    ("ANTIBIO", code),
                                    ("CMI", cmi or self.default_cmi),
                                    ("SENSI", sensi),
                                    ("DIAMETRE", antibiotic.findtext("diameter")),
                                )
                            )
                        else:
                            result["atb"].append(
                                (
                                    ("ANTIBIO", code),
                                    ("CMI", cmi or self.default_cmi),
                                    ("SENSI", sensi),
                                )
                            )
                    # Récupérer le numéro de lot de l'ATB
                    if (
                        test_order.findall("./test/result/value/ast/antibiotic")
                        and lot_number
                    ):
                        sample_results[lot_number] = {
                            "test": "lotNumber",
                            "resu": lot_number,
                        }

                    if not result["atb"]:
                        # identification de germes
                        code_test = "GERME_%s_%s" % (result["seq"], result["test"])
                        sample_results[code_test] = {
                            "test": code_test,
                            "resu": germe,
                            "analyseur": analyseur,
                        }

                        result["test"] = "IDENT_" + result["test"]
                        result["resu"] = {
                            "type_resultat": "identification_germes",
                            "germes": [],
                        }
                        result["seq"] = "0"
                        if result["test"] in sample_results:
                            result["resu"]["germes"] = sample_results[result["test"]][
                                "resu"
                            ].get("germes", [])
                        result["resu"]["germes"].append(
                            {
                                "id": germe,
                                "rang": int(test_order.findtext("isolateNumber")),
                            }
                        )
                    elif result["test"] in sample_results:
                        raise NotImplementedError(
                            "Plusieurs resultats pour un meme test avec ATB."
                        )
                    sample_results[result["test"]] = result

        results = {}
        for sample_id, data in samples_results.items():
            results.setdefault(sample_id, {"resultats": []})["resultats"].extend(
                list(data.values())
            )
        return results
